/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bpel.engine.listener;

import java.util.logging.Level;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.output.DOMOutputter;
import org.ow2.petals.bpel.engine.BPELEngine;
import org.ow2.petals.bpel.engine.message.JBIContextImpl;
import org.ow2.petals.bpel.engine.message.JBIExternalMessageImpl;
import org.ow2.petals.bpel.engine.util.Sender;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.UtilFactory;

import com.ebmwebsourcing.easybpel.model.bpel.impl.exception.UserDefinedException;
import com.ebmwebsourcing.easyviper.core.api.Core;
import com.ebmwebsourcing.easyviper.core.api.CoreException;
import com.ebmwebsourcing.easyviper.core.api.env.Receiver;

/**
 * This class implements the JBI listener for orchestra engine
 *
 * @author Nicolas SALATGE - eBMWebSourcing
 *
 */
public final class JBIListener extends AbstractJBIListener {


    private Core core;

    static private int currentIndReceiver = 0;

    /**
     * The default constructor
     */
    public JBIListener() {

    }

    /**
     * Retrieve the BPELProcessEnvironmentMap for further access to bpel process
     * environments
     *
     * @see org.ow2.petals.component.common.listener.AbstractListener#init()
     */
    @Override
    public void init() {
        this.core = ((BPELEngine) this.getComponent())
        .getCore();


    }

    /**
     * (non-Javadoc)
     *
     * @param exchange
     *            the exchange
     * @see org.ow2.petals.component.common.listener.AbstractJBIListener#onJBIMessage(org.ow2.petals.component.common.util.MessageExchangeWrapper)
     */
    @Override
    public boolean onJBIMessage(final Exchange exchange) {

        if(!exchange.isActiveStatus()) {
            this.getLogger().finest("Exchange is not in Active Status .... ");
            return true;
        }
        
        
        boolean isInOnly = exchange.isInOnlyPattern();

        try {

            final Sender sender = new Sender(this.getComponent().getContext(), this.getComponent().getChannel(), this.getLogger());
            final JBIContextImpl context = new JBIContextImpl(exchange, sender, this);
            final JBIExternalMessageImpl message = new JBIExternalMessageImpl();

            message.setEndpoint(exchange.getEndpointName());
            message.setService(exchange.getService());
            message.setOperationName(exchange.getOperation().getLocalPart());
            message.setContent(exchange.getInMessageContentAsDocument().getDocumentElement());
            


            //InOnly Pattern
            if(isInOnly){
                try {

                    exchange.setDoneStatus();
                    this.send(exchange);
                } catch (MessagingException e) {
                    //Never happen apparently
                    this.getLogger().severe(e.getMessage());
                    return false;
                }

            }

            if(JBIListener.currentIndReceiver >= this.core.getExternalEnvironment().getReceivers().size()) {
                JBIListener.currentIndReceiver = 0;
            }

            final Receiver receiver = this.core.getExternalEnvironment().getReceivers().get(JBIListener.currentIndReceiver);
            JBIListener.currentIndReceiver++;

            if (this.getLogger().isLoggable(Level.FINE)){
                this.getLogger().fine("Message accepted by receiver " + receiver.getName() + " :\n" + message);
                this.getLogger().fine("Invoked operation:  " + message.getOperationName());
            }

            synchronized(receiver) {
                receiver.accept(message, context);
            }

        } catch (final MessagingException e) {
            this.getLogger().finest("Problem during processing exchange" + e.getMessage());

            if(!isInOnly){

                exchange.setError(e);
                try {
                    this.send(exchange);
                } catch (MessagingException e1) {
                    // do nothing
                    this.getLogger().severe(e1.getMessage());
                }
            }

        } catch (final CoreException e) {
            this.getLogger().finest("Problem accepting message in BPEL Core " + e.getMessage());
            if(e instanceof UserDefinedException) {
                try {
                    
                    if(MEPConstants.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                        
                        Fault f = exchange.createFault();

                        Element jFault = (Element) ((UserDefinedException)e).getFault().getContent();
                        Document jdomFault = jFault.getDocument();

                        // convert jdom to dom
                        DOMOutputter converter = new DOMOutputter();
                        org.w3c.dom.Document domDocument = converter.output(jdomFault);

                        // set content
                        f.setContent(UtilFactory.getSourceUtil().createDOMSource(domDocument));

                        exchange.setFault(f);
                        try {
                            this.send(exchange);
                        } catch (MessagingException e1) {
                            // do nothing
                            this.getLogger().severe(e1.getMessage());
                        }
                    }
                } catch (MessagingException e1) {
                    // do nothing
                    this.getLogger().severe(e1.getMessage());
                } catch (JDOMException e1) {
                    // do nothing
                    this.getLogger().severe(e1.getMessage());
                }
            } else {
                if(MEPConstants.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                    try {
                        exchange.setFault(new Exception(e));
                        //                        this.send(exchange);
                    } catch (MessagingException e1) {
                        // do nothing
                        this.getLogger().severe(e1.getMessage());
                    }
                }
            }
        }
        return false;
    }



}

